@@ -7,6 +7,7 @@ use std::{io, mem};
 use nix::fcntl::{fcntl, OFlag};
 use nix::{libc, unistd};
 
+use crate::posix::flock;
 use crate::posix::ioctl::{self, SerialLines};
 use crate::posix::termios;
 use crate::{
@@ -17,8 +18,12 @@ use crate::{
 /// Convenience method for removing exclusive access from
 /// a fd and closing it.
 fn close(fd: RawFd) {
-    // remove exclusive access
+    // Remove exclusive access on best-effort. There is no documentation hinting at `TIOCEXCL`
+    // being cleared automatically so explicitly attempt it here.
     let _ = ioctl::tiocnxcl(fd);
+    // However, it's documented for `flock` that the file will be unlocked when all filedescriptors
+    // are `close()`d. So don't bother with releasing the flock - we're going to close the
+    // filedescriptor immediately.
 
     // On Linux and BSD, we don't need to worry about return
     // type as EBADF means the fd was never open or is already closed
@@ -47,7 +52,7 @@ fn close(fd: RawFd) {
 /// explicitly _skip_ an attempt to set the baud rate of the file descriptor
 /// that would otherwise happen via an `ioctl` command.
 ///
-/// ```
+/// ```no_run
 /// use serialport::{TTYPort, SerialPort};
 ///
 /// let (mut master, mut slave) = TTYPort::pair().expect("Unable to create ptty pair");
@@ -111,7 +116,7 @@ impl TTYPort {
     /// ## Errors
     ///
     /// * `NoDevice` if the device could not be opened. This could indicate that
-    ///    the device is already in use.
+    ///   the device is already in use.
     /// * `InvalidInput` if `path` is not a valid device name.
     /// * `Io` for any other error while opening or initializing the device.
     pub fn open(builder: &SerialPortBuilder) -> Result<TTYPort> {
@@ -130,6 +135,9 @@ impl TTYPort {
         // other applications that may have an exclusive port lock.
         ioctl::tiocexcl(fd.0)?;
 
+        // Also use flock to lock the port
+        flock::lock_exclusive(fd.0)?;
+
         let mut termios = MaybeUninit::uninit();
         nix::errno::Errno::result(unsafe { tcgetattr(fd.0, termios.as_mut_ptr()) })?;
         let mut termios = unsafe { termios.assume_init() };
@@ -192,10 +200,11 @@ impl TTYPort {
             baud_rate: builder.baud_rate,
         };
 
-        // Ignore setting DTR for pseudo terminals (indicated by baud_rate == 0).
+        // Ignore setting DTR for pseudo terminals. This might be indicated by baud_rate == 0, but
+        // as this is not always the case, just try on best-effort.
         if builder.baud_rate > 0 {
             if let Some(dtr) = builder.dtr_on_open {
-                port.write_data_terminal_ready(dtr)?;
+                let _ = port.write_data_terminal_ready(dtr);
             }
         }
 
@@ -215,7 +224,13 @@ impl TTYPort {
     /// If a port is exclusive, then trying to open the same device path again
     /// will fail.
     ///
-    /// See the man pages for the tiocexcl and tiocnxcl ioctl's for more details.
+    /// The tiocexcl ioctl is used to prevent other applications from opening
+    /// the port.
+    ///
+    /// `flock` is used to place an advisory lock, which prevents conflicts with
+    /// other applications using `flock`.
+    ///
+    /// See the man pages for the tiocexcl/tiocnxcl ioctl's and `flock` for more details.
     ///
     /// ## Errors
     ///
@@ -228,6 +243,15 @@ impl TTYPort {
         };
 
         setting_result?;
+
+        let flock_result = if exclusive {
+            flock::lock_exclusive(self.fd)
+        } else {
+            flock::lock_shared(self.fd)
+        };
+
+        flock_result?;
+
         self.exclusive = exclusive;
         Ok(())
     }
@@ -414,10 +438,20 @@ fn get_termios_speed(fd: RawFd) -> u32 {
 
 impl FromRawFd for TTYPort {
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        let flock_successful = flock::lock_exclusive(fd).is_ok();
+
+        // TODO: If we fail to get the exclusive lock, this probably means that
+        // another process is using the port, and we should return an error,
+        // instead of using the port in non-exclusive mode.
+        //
+        // This will require a breaking change, as this method currently can't fail.
+
+        let tiocexcl_successful = ioctl::tiocexcl(fd).is_ok();
+
         TTYPort {
             fd,
             timeout: Duration::from_millis(100),
-            exclusive: ioctl::tiocexcl(fd).is_ok(),
+            exclusive: tiocexcl_successful && flock_successful,
             // It is not trivial to get the file path corresponding to a file descriptor.
             // We'll punt on it and set it to `None` here.
             port_name: None,
